/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

import { IdentityAppsApiException } from "@wso2is/core/exceptions";
import { AlertInterface, AlertLevels, IdentifiableComponentInterface, SBACInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import {
    ContentLoader,
    EmphasizedSegment,
    EmptyPlaceholder,
    Hint,
    Link,
    PrimaryButton,
    Text
} from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, SyntheticEvent, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useDispatch } from "react-redux";
import { Dispatch } from "redux";
import { Dropdown, DropdownItemProps, DropdownProps, Form, Grid } from "semantic-ui-react";
import { ExtendedFeatureConfigInterface } from "../../../../configs/models";
import { APIResourcesConstants } from "../../../api-resources/constants";
import { patchScopesOfAPIResource, useScopesOfAPIResources } from "../../api";
import { AuthorizedAPIListItemInterface, AuthorizedPermissionListItemInterface } from "../../models";

/**
 * Prop-types for the API resources page component.
 */
interface ScopeFormInterface extends
    SBACInterface<ExtendedFeatureConfigInterface>, IdentifiableComponentInterface {
    /**
     * Application ID.
     */
    appId: string;
    /**
     * Currently selected API resource.
     */
    subscribedAPIResource: AuthorizedAPIListItemInterface;
    /**
     * Flag to check if the scopes are available for update.
     */
    isScopesAvailableForUpdate: boolean;
    /**
     * Update all authorized scopes.
     */
    updateAllAuthorizedScopes: (updatedScope: AuthorizedPermissionListItemInterface, removed: boolean) => void;
    /**
     * Bulk change all authorized scopes.
     */
    bulkChangeAllAuthorizedScopes: (scopes: AuthorizedPermissionListItemInterface[], removed: boolean) => void;
}

/**
 * API Resources listing page.
 *
 * @param props - Props injected to the component.
 * @returns API Resources Page component
 */
export const ScopeForm: FunctionComponent<ScopeFormInterface> = (
    props: ScopeFormInterface
): ReactElement => {

    const {
        appId,
        subscribedAPIResource,
        isScopesAvailableForUpdate,
        bulkChangeAllAuthorizedScopes,
        updateAllAuthorizedScopes,
        ["data-componentid"]: componentId
    } = props;

    const { t } = useTranslation();
    const dispatch: Dispatch = useDispatch();

    const [ isLoading, setIsLoading ] = useState<boolean>(false);
    const [ isUpdateButtonDisabled, setIsUpdateButtonDisabled ] = useState<boolean>(true);
    const [ dropdownOptions, setDropdownOptions ] = useState<DropdownItemProps[]>(null);
    const [ defaultDropdownValues, setDefaultDropdownValues ] = useState<string[]>(null);
    const [ previousSelectedScopes, setPreviousSelectedScopes ] 
        = useState<AuthorizedPermissionListItemInterface[]>(subscribedAPIResource.permissions);
    const [ selectedScopes, setSelectedScopes ] = useState<string[]>(null);
    const [ addedScopesIdentifiers, setAddedScopesIdentifiers ] = useState<string[]>([]);
    const [ removedScopesIdentifiers, setRemovedScopesIdentifiers ] = useState<string[]>([]);

    // TODO: Check if this API resource is fetched from the all GET API resources call.
    // If so, we can use that instead of making another call.
    const {
        data: allScopesOfAPIResource,
        isLoading: isAllScopesOfAPIResourceLoading,
        error: allScopesOfAPIResourceError
    } = useScopesOfAPIResources(subscribedAPIResource.apiId);

    /**
     * Check if the place holders should be shown.
     */
    const showPlaceHolders: boolean = isAllScopesOfAPIResourceLoading ||
        isLoading ||
        dropdownOptions === null || 
        defaultDropdownValues === null ||
        selectedScopes === null ||
        allScopesOfAPIResourceError !=  null ;

    /**
     * The following useEffect is used to handle if any error occurs while fetching scopes of the API resource.
     */
    useEffect(() => {
        if (allScopesOfAPIResourceError) {
            switch (allScopesOfAPIResourceError.response?.data?.code) {
                case APIResourcesConstants.UNAUTHORIZED_ACCESS:
                    dispatch(addAlert<AlertInterface>({
                        description: t("extensions:develop.apiResource.notifications.getAPIResources" +
                            ".unauthorizedError.description"),
                        level: AlertLevels.ERROR,
                        message: t("extensions:develop.apiResource.notifications.getAPIResources" +
                            ".unauthorizedError.message")
                    }));

                    break;

                default:
                    dispatch(addAlert<AlertInterface>({
                        description: t("extensions:develop.apiResource.notifications.getAPIResources" +
                            ".genericError.description"),
                        level: AlertLevels.ERROR,
                        message: t("extensions:develop.apiResource.notifications.getAPIResources" +
                            ".genericError.message")
                    }));
            }

        }
    }, [ allScopesOfAPIResourceError ]);

    /**
     * Initialize the dropdown options.
     */
    useEffect(() => {
        const options: DropdownItemProps[] = [];

        allScopesOfAPIResource?.forEach((scope: AuthorizedPermissionListItemInterface) => {
            options.push({
                key: scope.name,
                text: scope.displayName,
                value: scope.name
            });
        });
        setDropdownOptions(options);
    }, [ allScopesOfAPIResource ]);

    /**
     * Intialize the default dropdown values.
     */
    useEffect(() => {
        const defaultValues: string[] = [];

        subscribedAPIResource.permissions.forEach((scope: AuthorizedPermissionListItemInterface) => {
            defaultValues.push(scope.name);
        });
        setDefaultDropdownValues(defaultValues);
        setSelectedScopes(defaultValues);
    }, [ subscribedAPIResource ]);

    /**
     * Handles patch request to update the scopes of the API resource.
     *
     * @param apiResourceId - API resource ID.
     */
    const updateScopesOfAPIResource = (): void => {

        setIsLoading(true);
        patchScopesOfAPIResource(appId, subscribedAPIResource?.apiId, addedScopesIdentifiers, removedScopesIdentifiers)
            .then(() => {
                dispatch(addAlert<AlertInterface>({
                    description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                        ".sections.apiSubscriptions.notifications.patchScopes.success.description" ),
                    level: AlertLevels.SUCCESS,
                    message: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                        ".sections.apiSubscriptions.notifications.patchScopes.success.message")
                }));

                // Update the added and removed scopes.
                setAddedScopesIdentifiers([]);
                setRemovedScopesIdentifiers([]);

                // Update the default dropdown values.
                setDefaultDropdownValues(selectedScopes);

                // Set the update button to disabled.
                setIsUpdateButtonDisabled(true);
            })
            .catch((error: IdentityAppsApiException) => {
                switch(error?.code) {
                    case APIResourcesConstants.UNAUTHORIZED_ACCESS:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.apiSubscriptions.notifications.patchScopes.unauthorizedError.description" ),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.apiSubscriptions.notifications.patchScopes.unauthorizedError.message" )
                        }));

                        break;

                    default:
                        dispatch(addAlert<AlertInterface>({
                            description: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.apiSubscriptions.notifications.patchScopes.genericError.description" ),
                            level: AlertLevels.ERROR,
                            message: t("extensions:develop.applications.edit.sections.apiAuthorization" +
                                ".sections.apiSubscriptions.notifications.patchScopes.genericError.message" )
                        }));
                }
            })
            .finally(() => setIsLoading(false));
    };

    /**
     * Placeholder for the scope form.
     */
    const getPlaceholders = (): ReactElement => {
        if (
            isAllScopesOfAPIResourceLoading ||
            selectedScopes === null ||
            dropdownOptions === null || 
            defaultDropdownValues === null || 
            isLoading
        ) {
            return <ContentLoader inline="centered" active />;
        } else if (allScopesOfAPIResourceError) {
            return (
                <EmptyPlaceholder
                    title={ t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                        ".apiSubscriptions.placeHolderTexts.errorText.title") }
                    subtitle={ [ t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                        ".apiSubscriptions.placeHolderTexts.errorText.subtitles.1"),
                    t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                        ".apiSubscriptions.placeHolderTexts.errorText.subtitles.0") ] }
                    data-componentid={ `${componentId}-all-empty-placeholder-icon` }
                />
            );
        } else {
            return null;
        }
    };

    /**
     * The following function is used to handle the dropdown change.
     */
    const handleDropdownChange = (event: SyntheticEvent<HTMLElement, Event>, data: DropdownProps): void => {
        const scopes: string[] = data.value as string[];

        setSelectedScopes(scopes);

        // Set the update button to disabled if the scopes are not changed.
        if (scopes.toString() !== defaultDropdownValues.toString()) {
            setIsUpdateButtonDisabled(false);
        } else {
            setIsUpdateButtonDisabled(true);
        }

        // Handles if a scope is added.
        if (scopes.length >= previousSelectedScopes.length) {
            const scopeObject: AuthorizedPermissionListItemInterface = allScopesOfAPIResource?.find(
                (scopeObject: AuthorizedPermissionListItemInterface) => 
                    scopeObject.name === scopes[scopes.length - 1]);
            
            // Update the all authorized scopes.
            if (scopeObject) {
                updateAllAuthorizedScopes(scopeObject, false);
                setPreviousSelectedScopes([ ...previousSelectedScopes, scopeObject ]);
            }

            // Set the added scopes identifiers.
            if (!defaultDropdownValues.includes(scopeObject.name)) {
                setAddedScopesIdentifiers([ ...addedScopesIdentifiers, scopeObject.name ]);
            }

            // Remove the scope from the removed scopes identifiers if it is already added.
            if (removedScopesIdentifiers.includes(scopeObject.name)) {
                setRemovedScopesIdentifiers(removedScopesIdentifiers.filter(
                    (scopeIdentifier: string) => scopeIdentifier !== scopeObject.name));
            }
        } else {
            // Handles if a scope is removed.
            const scopeObject: AuthorizedPermissionListItemInterface = previousSelectedScopes?.filter(
                (scope: AuthorizedPermissionListItemInterface) =>
                    !scopes.includes(scope.name))[0];
            
            // Update the all authorized scopes.
            if (scopeObject) {
                updateAllAuthorizedScopes(scopeObject, true);
                setPreviousSelectedScopes(previousSelectedScopes.filter(
                    (scope: AuthorizedPermissionListItemInterface) => scope.name !== scopeObject.name));
            }

            // Set the removed scopes identifiers.
            if (defaultDropdownValues.includes(scopeObject.name)) {
                setRemovedScopesIdentifiers([ ...removedScopesIdentifiers, scopeObject.name ]);
            }

            // Remove the scope from the added scopes identifiers if it is already added.
            if (addedScopesIdentifiers.includes(scopeObject.name)) {
                setAddedScopesIdentifiers(addedScopesIdentifiers.filter(
                    (scopeIdentifier: string) => scopeIdentifier !== scopeObject.name));
            }
        }
    };

    /**
     * Handle bulk dropdown change.
     */
    const handleBulkDropdownChange = (scopeObjectList: AuthorizedPermissionListItemInterface[], 
        remove: boolean): void => {
        const scopeNameList: string[] = 
            scopeObjectList.map((scope: AuthorizedPermissionListItemInterface) => scope.name);

        // Update the all authorized scopes.
        bulkChangeAllAuthorizedScopes(scopeObjectList, remove);

        // Handles `Select None` option.
        if (remove) {
            // Set the update button to disabled if the scopes are not changed.
            if (defaultDropdownValues.toString() !== [].toString()) {
                setIsUpdateButtonDisabled(false);
            } else {
                setIsUpdateButtonDisabled(true);
            }

            setPreviousSelectedScopes([]);

            setSelectedScopes([]);
            // Set the removed scopes identifiers.
            setRemovedScopesIdentifiers(defaultDropdownValues);

            // Remove the scope from the added scopes identifiers if it is already added.
            setAddedScopesIdentifiers([]);
        } else {
            // Handles `Select All` option.
            // Set the update button to disabled if the scopes are not changed.
            if (defaultDropdownValues.toString() !== scopeNameList.toString()) {
                setIsUpdateButtonDisabled(false);
            } else {
                setIsUpdateButtonDisabled(true);
            }

            setPreviousSelectedScopes(scopeObjectList);

            setSelectedScopes(scopeNameList);
            // Set the added scopes identifiers.
            setAddedScopesIdentifiers(scopeNameList.filter(
                (scopeName: string) => !defaultDropdownValues.includes(scopeName)));

            // Remove the scope from the removed scopes identifiers if it is already added.
            setRemovedScopesIdentifiers([]);
        }     
    };

    return (
        <EmphasizedSegment padded="very">
            {
                showPlaceHolders
                    ? getPlaceholders()
                    : (
                        <Grid>
                            <Grid.Row columns={ 1 }>
                                <Grid.Column>
                                    <Form>
                                        <Form.Field>
                                            <Grid>
                                                <Grid.Row columns={ 2 }>
                                                    <Grid.Column floated="left" stretched>
                                                        <label>
                                                            { t("extensions:develop.applications.edit.sections." +
                                                                "apiAuthorization.sections.apiSubscriptions." +
                                                                "scopesSection.label" ) }
                                                        </label>
                                                    </Grid.Column>
                                                    <Grid.Column floated="right" textAlign="right">
                                                        {
                                                            isScopesAvailableForUpdate && (
                                                                <Text className="mb-0" muted subHeading size={ 12 }>
                                                                    <Link
                                                                        external={ false }
                                                                        className="mr-1"
                                                                        onClick={ () => 
                                                                            handleBulkDropdownChange( 
                                                                                allScopesOfAPIResource 
                                                                                    ? allScopesOfAPIResource
                                                                                    : [], 
                                                                                false ) }
                                                                    >
                                                                        { 
                                                                            t("extensions:develop.applications.edit." +
                                                                            "sections.apiAuthorization.sections." +
                                                                            "apiSubscriptions.scopesSection.selectAll" )
                                                                        }
                                                                    </Link>
                                                                    | 
                                                                    <Link
                                                                        external={ false }
                                                                        className="ml-1"
                                                                        onClick={ () => handleBulkDropdownChange(
                                                                            allScopesOfAPIResource 
                                                                                ? allScopesOfAPIResource
                                                                                : [],
                                                                            true ) }
                                                                    >
                                                                        { 
                                                                            t("extensions:develop.applications.edit." +
                                                                            "sections.apiAuthorization.sections." +
                                                                            "apiSubscriptions.scopesSection." +
                                                                            "selectNone" )
                                                                        }
                                                                    </Link>
                                                                </Text>
                                                            )
                                                        }
                                                    </Grid.Column>
                                                </Grid.Row>
                                            </Grid>
                                            <Dropdown
                                                placeholder={ t("extensions:develop.applications.edit.sections." +
                                                    "apiAuthorization.sections.apiSubscriptions.scopesSection." + 
                                                    "placeholder") }
                                                fluid
                                                multiple
                                                search
                                                selection
                                                options={
                                                    dropdownOptions
                                                        ? dropdownOptions
                                                        : []
                                                }
                                                defaultValue={ defaultDropdownValues }
                                                value={ selectedScopes }
                                                onChange={ handleDropdownChange }
                                                disabled={ !isScopesAvailableForUpdate }
                                            />
                                            <Hint>
                                                { t("extensions:develop.applications.edit.sections." +
                                                    "apiAuthorization.sections.apiSubscriptions.scopesSection." + 
                                                    "hint") }
                                            </Hint>
                                        </Form.Field>
                                    </Form>
                                </Grid.Column>
                            </Grid.Row>
                            {
                                isScopesAvailableForUpdate && (
                                    <Grid.Row columns={ 1 }>
                                        <Grid.Column>
                                            <PrimaryButton
                                                basic
                                                size="small"
                                                tabIndex={ 6 }
                                                onClick={ updateScopesOfAPIResource }
                                                data-componentid={ `${componentId}-update-btn` }
                                                disabled={ isUpdateButtonDisabled }
                                            >
                                                { t("extensions:develop.applications.edit.sections.apiAuthorization." +
                                                    "sections.policySection.buttons.update") }
                                            </PrimaryButton>
                                        </Grid.Column>
                                    </Grid.Row>
                                )
                            }
                        </Grid>
                    )
            }
        </EmphasizedSegment>
    );
};

/**
 * Default props for the component.
 */
ScopeForm.defaultProps = {
    "data-componentid": "permission-form"
};
