/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 LLC. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content.
 */

import { AlertInterface, AlertLevels, IdentifiableComponentInterface, LinkInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import { Field, FormValue, Forms, useTrigger } from "@wso2is/forms";
import { ContentLoader, Heading, Hint, LinkButton, PrimaryButton, Text } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, SyntheticEvent, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useDispatch } from "react-redux";
import { Dispatch } from "redux";
import { Dropdown, DropdownItemProps, DropdownProps, Grid, Modal } from "semantic-ui-react";
import { useAPIResources } from "../../../../api-resources/api";
import { APIResourcesConstants } from "../../../../api-resources/constants";
import { APIResourceInterface, APIResourcePermissionInterface } from "../../../../api-resources/models";
import { AuthorizedAPIListItemInterface } from "../../../models";

interface AuthorizeAPIResourcePropsInterface extends IdentifiableComponentInterface {
    /**
     * List of subscribed API Resources
     */
    subscribedAPIResourcesListData: AuthorizedAPIListItemInterface[],
    /**
     * Close the wizard.
     */
    closeWizard: () => void;
    /**
     * Invoke authorized API resource creation.
     */
    handleCreateAPIResource: (apiId: string, scopes: string[], callback: ()=>void) => void;
}

/**
 * API resource wizard.
 * TODO: Add a message box to show why authorized scopes field is disabled.
 */
export const AuthorizeAPIResource: FunctionComponent<AuthorizeAPIResourcePropsInterface> = (
    props: AuthorizeAPIResourcePropsInterface
): ReactElement => {

    const {
        subscribedAPIResourcesListData,
        closeWizard,
        handleCreateAPIResource,
        ["data-componentid"]: componentId
    } = props;

    const { t } = useTranslation();
    const dispatch: Dispatch = useDispatch();
    const [ submitForm, setSubmitForm ] = useTrigger();

    const [ isSubmitting, setIsSubmitting ] = useState<boolean>(false);
    const [ isAPIResourcesListLoading, setIsAPIResourcesListLoading ] = useState<boolean>(false);
    const [ apiCallNextAfterValue, setAPICallNextAfterValue ] = useState<string>(null);
    const [ selectedAPIResourceId, setSelectedAPIResourceId ] = useState<string>(null);
    const [ authorizedScopes, setAuthorizedScopes ] = useState<string[]>([]);
    const [ allAPIResourcesDropdownOptions, setAllAPIResourcesDropdownOptions ] = useState<DropdownItemProps[]>([]);
    const [ scopesDropdownOptions, setScopesDropdownOptions ] = useState<DropdownItemProps[]>([]);
    const [ isScopesDropdownLoading, setIsScopesDropdownLoading ] = useState<boolean>(false);

    const {
        data: currentAPIResourcesListData,
        isLoading: iscurrentAPIResourcesListLoading,
        error: currentAPIResourcesFetchRequestError,
        mutate: mutatecurrentAPIResourcesList
    } = useAPIResources(apiCallNextAfterValue);

    /**
     * The following useEffect is used to handle if any error occurs while fetching API resources.
     */
    useEffect(() => {
        if (currentAPIResourcesFetchRequestError) {
            dispatch(addAlert<AlertInterface>({
                description: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                    ".apiSubscriptions.notifications.createAuthorizedAPIResource.initialError" + 
                    ".description"),
                level: AlertLevels.ERROR,
                message: t("extensions:develop.applications.edit.sections.apiAuthorization.sections" +
                    ".apiSubscriptions.notifications.createAuthorizedAPIResource.initialError.message")
            }));
            closeWizard();
        }
    }, [ currentAPIResourcesFetchRequestError ]);

    /**
     * Assign all the API resources to the dropdown options if the after value is not null. 
     */ 
    useEffect(() => {
        if (!isAPIResourcesListLoading) {
            setIsAPIResourcesListLoading(true);
        }

        let afterValue: string;

        if (currentAPIResourcesListData) {
            const filteredDropdownItemOptions: DropdownItemProps[] = 
            (currentAPIResourcesListData?.apiResources.reduce(function(filtered: DropdownItemProps[], 
                apiResource: APIResourceInterface) {
 
                const isCurrentAPIResourceSubscribed: boolean = subscribedAPIResourcesListData?.length === 0 
                    || !subscribedAPIResourcesListData?.some((subscribedAPIResource: AuthorizedAPIListItemInterface) =>
                        subscribedAPIResource.apiIdentifier === apiResource.identifier);

                if (isCurrentAPIResourceSubscribed)
                {
                    const isCurrentAPIResourceAlreadyAdded: boolean = allAPIResourcesDropdownOptions.length === 0
                        || !allAPIResourcesDropdownOptions?.some(
                            (dropdownOption: DropdownItemProps) => dropdownOption.key === apiResource.id);

                    if (isCurrentAPIResourceAlreadyAdded) 
                    {
                        filtered.push({
                            key: apiResource.id,
                            text: apiResource.displayName,
                            value: apiResource.id
                        });
                    }
                }

                return filtered;
            }, []));

            setAllAPIResourcesDropdownOptions( [ 
                ...allAPIResourcesDropdownOptions,
                ...filteredDropdownItemOptions ? filteredDropdownItemOptions : []
            ]);

            if (currentAPIResourcesListData?.links?.length > 0) {
                currentAPIResourcesListData?.links?.forEach((value: LinkInterface) => {
                    if (value.rel === APIResourcesConstants.AFTER_REL) {
                        afterValue = value.href.split(`${APIResourcesConstants.AFTER_REL}=`)[1];
                        setAPICallNextAfterValue(afterValue);
                    }
                });

                mutatecurrentAPIResourcesList();
            } else {
                setAPICallNextAfterValue(null);
                setIsAPIResourcesListLoading(false);
            }
        }
    }, [ currentAPIResourcesListData ]);

    /**
     * Assign scopes to the scopes dropdown options.
     */
    useEffect(() => {
        setIsScopesDropdownLoading(true);
        if (selectedAPIResourceId) {
            const selectedAPIResource: APIResourceInterface = currentAPIResourcesListData?.apiResources.find(
                (apiResource: APIResourceInterface) => {
                    return apiResource.id === selectedAPIResourceId;
                }
            );

            setScopesDropdownOptions(selectedAPIResource?.permissions.map((scope: APIResourcePermissionInterface) => {
                return {
                    key: scope.name,
                    text: scope.displayName,
                    value: scope.name
                };
            }));
            setAuthorizedScopes([]);
        }
        setIsScopesDropdownLoading(false);
    }, [ selectedAPIResourceId ]);

    /**
     * Set the selected API resource ID.
     */
    const handleAPIResourceDropdownChange = (isPure: boolean,values: Map<string, FormValue>) => 
        setSelectedAPIResourceId(values.get("apiResource").toString());

    /**
     * Handle the scopes dropdown change.
     */
    const handleScopesDropdownChange = (event: SyntheticEvent<HTMLElement>, data: DropdownProps) => {
        const scopes: string[] = data.value as string[];

        // Set the authorized scopes that are only in the selected API resource.
        setAuthorizedScopes(data.options.filter((option: DropdownItemProps) => {
            if (scopes.includes(option.value.toString())) {
                return option; 
            }})
            .map((option: DropdownItemProps) => option.value.toString()));
    };

    /**
     * Handle the form submission.
     */
    const handleFormSubmit = () => {
        setIsSubmitting(true);
        handleCreateAPIResource(selectedAPIResourceId, authorizedScopes, callBack);
    };

    /**
     * Callback after the API resource is authorized.
     */
    const callBack = () => {
        setIsSubmitting(false);
        setAllAPIResourcesDropdownOptions([]);
    };

    return (
        <Modal
            data-testid={ componentId }
            open={ true }
            className="wizard api-resource-create-wizard"
            dimmer="blurring"
            size="small"
            onClose={ closeWizard }
            closeOnDimmerClick={ false }
            closeOnEscape
        >
            <Modal.Header className="wizard-header">
                { t("extensions:develop.applications.edit.sections.apiAuthorization.sections.apiSubscriptions." +
                    "wizards.authorizeAPIResource.title") }
                <Heading as="h6">
                    { t("extensions:develop.applications.edit.sections.apiAuthorization.sections.apiSubscriptions." +
                        "wizards.authorizeAPIResource.subTitle") }
                </Heading>
            </Modal.Header>
            <Modal.Content className="content-container" scrolling>
                {
                    iscurrentAPIResourcesListLoading || isAPIResourcesListLoading
                        ? <ContentLoader inline="centered" active /> 
                        : (<Forms
                            data-componentid={ `${componentId}-form` }
                            uncontrolledForm={ false }
                            onChange={ handleAPIResourceDropdownChange }
                            onSubmit={ handleFormSubmit }
                            submitState={ submitForm }  
                        >
                            <Grid>
                                <Grid.Row columns={ 1 }>
                                    <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 12 }>
                                        <Field
                                            data-componentid={ `${componentId}-api` }
                                            label={ t("extensions:develop.applications.edit.sections." +
                                                "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                                "authorizeAPIResource.fields.apiResource.label") }
                                            placeholder={ t("extensions:develop.applications.edit.sections." +
                                                "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                                "authorizeAPIResource.fields.apiResource.placeholder") }
                                            type="dropdown"
                                            name="apiResource"
                                            key="apiResource"
                                            children={ allAPIResourcesDropdownOptions }
                                            tabIndex={ 3 }
                                            required={ true }
                                            requiredErrorMessage={ t("extensions:develop.applications.edit.sections." +
                                                "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                                "authorizeAPIResource.fields.apiResource.requiredErrorMessage") }
                                            search
                                            selection
                                            scrolling
                                        />
                                    </Grid.Column>
                                </Grid.Row>
                                <Grid.Row columns={ 1 }>
                                    <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 12 }>
                                        <label>
                                            <Text size={ 13 } className="mb-0" muted={ !selectedAPIResourceId }>
                                                { t("extensions:develop.applications.edit.sections." +
                                                    "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                                    "authorizeAPIResource.fields.scopes.label") }
                                            </Text>
                                        </label>
                                        <Dropdown
                                            placeholder={ t("extensions:develop.applications.edit.sections." +
                                                "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                                "authorizeAPIResource.fields.scopes.placeholder") }
                                            fluid
                                            multiple
                                            search
                                            selection
                                            onChange={ handleScopesDropdownChange }
                                            loading={ isScopesDropdownLoading }
                                            options={ scopesDropdownOptions }
                                            disabled={ !selectedAPIResourceId }
                                        />
                                        <Hint disabled={ !selectedAPIResourceId }>
                                            { t("extensions:develop.applications.edit.sections." +
                                                "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                                "authorizeAPIResource.fields.scopes.hint") }
                                        </Hint>
                                    </Grid.Column>
                                </Grid.Row>
                            </Grid>
                        </Forms>)
                }
            </Modal.Content>
            <Modal.Actions>
                <Grid>
                    <Grid.Row column={ 1 }>
                        <Grid.Column mobile={ 8 } tablet={ 8 } computer={ 8 }>
                            <LinkButton
                                tabIndex={ 5 }
                                data-testid={ `${componentId}-cancel-button` }
                                floated="left"
                                onClick={ closeWizard }
                            >
                                { t("extensions:develop.applications.edit.sections." +
                                        "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                        "authorizeAPIResource.buttons.cancel") }
                            </LinkButton>
                        </Grid.Column>
                        <Grid.Column mobile={ 8 } tablet={ 8 } computer={ 8 }>
                            <PrimaryButton
                                tabIndex={ 6 }
                                data-testid={ `${componentId}-finish-button` }
                                floated="right"
                                onClick={ setSubmitForm }
                                loading={ isSubmitting }
                            >
                                { t("extensions:develop.applications.edit.sections." +
                                        "apiAuthorization.sections.apiSubscriptions.wizards." + 
                                        "authorizeAPIResource.buttons.finish") }
                            </PrimaryButton>
                        </Grid.Column>
                    </Grid.Row>
                </Grid>
            </Modal.Actions>
        </Modal>
    );
};

AuthorizeAPIResource.defaultProps = {
    "data-componentid": "authorize-api-resource"
};
